/**************************************************************************//**
 * @file     main.c

 * @brief    This sample code use DSP_FFT to get main frequency from Line in.
 *
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "NUC505Series.h"
#include "arm_math.h"
#include "arm_const_structs.h"
#include "config.h"
/*---------------------------------------------------------------------------*/
/* Define                                                                    */
/*---------------------------------------------------------------------------*/
#define SampleRate 8000
#define VolTH 200000
/*---------------------------------------------------------------------------*/
/* Global variables                                                          */
/*---------------------------------------------------------------------------*/
int16_t PcmRxBuff[2][BUFF_LEN * 2] = {0};
int16_t PcmTxBuff[2][BUFF_LEN * 2] = {0};
float32_t RawData[BUFF_LEN * 2] = {0};  //even is real part, odd is imaginary part
float32_t FFTOutput[BUFF_LEN] = {0}, maxValue = 0, MainFreq = 0;
volatile uint8_t DataReady;
uint32_t  maxIndex = 0;
/*---------------------------------------------------------------------------*/
/* Functions                                                                 */
/*---------------------------------------------------------------------------*/
void demo_LineIn(void)
{
    uint32_t i;

    /* Setting Right Line In Channel */
    SYS->GPD_MFPL  = (SYS->GPD_MFPL & (~SYS_GPD_MFPL_PD4MFP_Msk)) | SYS_GPD_MFPL_PD4MFP_RLINEIN;
    SYS_SetSharedPinType(SYS_PORT_D, 4, 0, 0);

    /* IIC Configure Step without PLL: */
    /* Add MCLK(256*Fs) in. */

    I2S_SET_INTERNAL_CODEC(I2S, 0x08, 0x1F);    //Mute headphone of Left channel
    I2S_SET_INTERNAL_CODEC(I2S, 0x09, 0x1F);    //Mute headphone of Right channel
    I2S_SET_INTERNAL_CODEC(I2S, 0x10, 0x0F);    //Mute the ADC Left channel volume
    I2S_SET_INTERNAL_CODEC(I2S, 0x11, 0x0F);    //Mute the ADC Right channel volume
    I2S_SET_INTERNAL_CODEC(I2S, 0x12, 0x0F);    //Mute the ADC Side tone volume

    I2S_SET_INTERNAL_CODEC(I2S, 0x02, 0xC0);    //Set CODEC slave

    I2S_SET_INTERNAL_CODEC(I2S, 0x01, 0x80);    //Digital Part Enable
    I2S_SET_INTERNAL_CODEC(I2S, 0x0F, 0xF0);    //Enable Analog Part
    I2S_SET_INTERNAL_CODEC(I2S, 0x0E, 0x00);    //ADC input select Line in

    I2S_SET_INTERNAL_CODEC(I2S, 0x0B, 0xF3);    //Analog Part Enable
    I2S_SET_INTERNAL_CODEC(I2S, 0x0D, 0x31);    //Biasing enable
    I2S_SET_INTERNAL_CODEC(I2S, 0x0B, 0xE3);

    for (i = 0; i < 15; i++) //Delay 1.5s~2.5s
        CLK_SysTickDelay(100000);

    I2S_SET_INTERNAL_CODEC(I2S, 0x0A, 0x09);
    I2S_SET_INTERNAL_CODEC(I2S, 0x0B, 0xF0);
    I2S_SET_INTERNAL_CODEC(I2S, 0x00, 0xD0);    //ADC digital enabled
    CLK_SysTickDelay(100000);   //Delay 100mS

    I2S_SET_INTERNAL_CODEC(I2S, 0x08, 0x06);    //Un-mute Headphone and set volume
    I2S_SET_INTERNAL_CODEC(I2S, 0x09, 0x06);    //Un-mute Headphone and set volume
    I2S_SET_INTERNAL_CODEC(I2S, 0x10, 0x08);    //Un-Mute the ADC Left channel volume
    I2S_SET_INTERNAL_CODEC(I2S, 0x11, 0x08);    //Un-Mute the ADC Right channel volume
    //I2S_SET_INTERNAL_CODEC(I2S, 0x12, 0x00);    //Un-Mute the ADC Side tone volume

    /* If Fs is changed, please Mute Headphone First and soft reset digital part after MCLK is stable. */
}
void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Enable  XTAL */
    CLK->PWRCTL |= CLK_PWRCTL_HXTEN_Msk;

    CLK_SetCoreClock(FREQ_96MHZ);

    /* PCLK divider */
    CLK_SetModuleClock(PCLK_MODULE, NULL, 0);
}

void UART0_Init(void)
{
    /* Enable UART0 Module clock */
    CLK_EnableModuleClock(UART0_MODULE);
    /* UART0 module clock from EXT */
    CLK_SetModuleClock(UART0_MODULE, CLK_UART0_SRC_EXT, 0);
    /* Reset IP */
    SYS_ResetModule(UART0_RST);
    /* Configure UART0 and set UART0 Baud-rate */
    UART_Open(UART0, 115200);
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Configure multi-function pins for UART0 RXD and TXD */
    SYS->GPB_MFPL  = (SYS->GPB_MFPL & (~SYS_GPB_MFPL_PB0MFP_Msk)) | SYS_GPB_MFPL_PB0MFP_UART0_TXD;
    SYS->GPB_MFPL  = (SYS->GPB_MFPL & (~SYS_GPB_MFPL_PB1MFP_Msk)) | SYS_GPB_MFPL_PB1MFP_UART0_RXD;
}

void I2S_Init(void)
{
    /* Enable I2S Module clock */
    CLK_EnableModuleClock(I2S_MODULE);

    /* Please set APLL for difference sampling rate */
    /* APLL = 49152031Hz for 8k, 12k, 16k, 24k, 32k, 48k, and 96k sampling rate */
    CLK_SET_APLL(CLK_APLL_49152031);
    /* APLL = 45158425Hz for 11025, 22050, and 44100 sampling rate */
    //CLK_SET_APLL(CLK_APLL_45158425);
    /* I2S module clock from APLL */
    CLK_SetModuleClock(I2S_MODULE, CLK_I2S_SRC_APLL, 1);
    /* Reset IP */
    SYS_ResetModule(I2S_RST);
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Configure multi-function pins for I2S */
    // GPC[8]  = MCLK
    // GPC[9]  = DIN
    // GPC[10] = DOUT
    // GPC[11] = LRCLK
    // GPC[12] = BCLK
    SYS->GPC_MFPH  = (SYS->GPC_MFPH & (~SYS_GPC_MFPH_PC8MFP_Msk)) | SYS_GPC_MFPH_PC8MFP_I2S_MCLK;
    SYS->GPC_MFPH  = (SYS->GPC_MFPH & (~SYS_GPC_MFPH_PC9MFP_Msk)) | SYS_GPC_MFPH_PC9MFP_I2S_DIN;
    SYS->GPC_MFPH  = (SYS->GPC_MFPH & (~SYS_GPC_MFPH_PC10MFP_Msk)) | SYS_GPC_MFPH_PC10MFP_I2S_DOUT;
    SYS->GPC_MFPH  = (SYS->GPC_MFPH & (~SYS_GPC_MFPH_PC11MFP_Msk)) | SYS_GPC_MFPH_PC11MFP_I2S_LRCLK;
    SYS->GPC_MFPH  = (SYS->GPC_MFPH & (~SYS_GPC_MFPH_PC12MFP_Msk)) | SYS_GPC_MFPH_PC12MFP_I2S_BCLK;
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void)
{
    uint8_t ifftFlag = 0, doBitReverse = 1;

    /* Init System, IP clock and multi-function I/O */
    SYS_Init();

    /* Init UART0 to 115200-8n1 for print message */
    UART0_Init();

    /* Init I2S, IP clock and multi-function I/O */
    I2S_Init();

    printf("+-----------------------------------------------------------+\n");
    printf("|         Fourier analysis using M4 DSP Sample Code         |\n");
    printf("|         audio data from LINE-IN (internal CODEC)          |\n");
    printf("+-----------------------------------------------------------+\n");
    printf("Sample rate is 8 kHz\n");
    printf("NOTE: Need 3.5 mm Male to Male Cable.\n\n");
    printf("The audio frequency is as follows: \n");

    /* Master mode, 16-bit word width, stereo mode, I2S format. Set TX and RX FIFO threshold to middle value. */
    /* Other sampling rate please change APLL clock setting in I2S_Init() */
    I2S_Open(I2S, I2S_MODE_MASTER, SampleRate, I2S_DATABIT_16, I2S_STEREO, I2S_FORMAT_I2S, I2S_ENABLE_INTERNAL_CODEC);

    /* Open MCLK */
    I2S_EnableMCLK(I2S, SampleRate * 256);

    I2S_SET_TX_TH_LEVEL(I2S, I2S_FIFO_TX_LEVEL_WORD_15);
    I2S_SET_RX_TH_LEVEL(I2S, I2S_FIFO_RX_LEVEL_WORD_16);

    /* Tx Start Address */
    I2S_SET_TXDMA_STADDR(I2S, (uint32_t) &PcmTxBuff[0]);
    /* Tx Threshold Address */
    I2S_SET_TXDMA_THADDR(I2S, (uint32_t) &PcmTxBuff[0][BUFF_LEN * 2 - 2]);
    /* Tx End Address */
    I2S_SET_TXDMA_EADDR(I2S, (uint32_t) &PcmTxBuff[1][BUFF_LEN * 2 - 2]);

    /* Rx Start Address */
    I2S_SET_RXDMA_STADDR(I2S, (uint32_t) &PcmRxBuff[0]);
    /* Rx Threshold Address */
    I2S_SET_RXDMA_THADDR(I2S, (uint32_t) &PcmRxBuff[0][BUFF_LEN * 2 - 2]);
    /* Rx End Address */
    I2S_SET_RXDMA_EADDR(I2S, (uint32_t) &PcmRxBuff[1][BUFF_LEN * 2 - 2]);

    /* Open Rx DMA Enable */
    I2S_ENABLE_RXDMA(I2S);

    /* Audio data input from Line in */
    demo_LineIn();

    /* Clear Interrupt Status */
    I2S_CLR_INT_FLAG(I2S, I2S_STATUS_LZCIF_Msk | I2S_STATUS_RZCIF_Msk | I2S_STATUS_TXOVIF_Msk | I2S_STATUS_TXUDIF_Msk | I2S_STATUS_RXOVIF_Msk | I2S_STATUS_RXUDIF_Msk | I2S_STATUS_TDMATIF_Msk |
                     I2S_STATUS_TDMAEIF_Msk | I2S_STATUS_RDMATIF_Msk | I2S_STATUS_RDMAEIF_Msk);
    /* Rx Enable */
    I2S_ENABLE_RX(I2S);
    NVIC_EnableIRQ(I2S_IRQn);
    I2S_EnableInt(I2S, (I2S_IEN_RDMATIEN_Msk | I2S_IEN_RDMAEIEN_Msk));

    /* This sample code only get audio data from right channel */
    while (1) {
        if (DataReady == 1) {
            /* Process the data through the CFFT/CIFFT module */
            arm_cfft_f32(&arm_cfft_sR_f32_len1024, RawData, ifftFlag, doBitReverse);
            /* Process the data through the Complex Magnitude Module for calculating the magnitude at each bin */
            arm_cmplx_mag_f32(RawData, FFTOutput, BUFF_LEN / 2);
            /* Calculates maxValue and returns corresponding BIN value : find maximum frequency */
            arm_max_f32(FFTOutput, BUFF_LEN / 2, &maxValue, &maxIndex);

            /* Check sound */
            if (maxValue < VolTH)
                printf("Silence... \n");
            else {
                /* Main frequency  calculation, the resolution is sampleRate / fftSize */
                MainFreq = maxIndex * SampleRate / BUFF_LEN;
                printf("%0.1f Hz\n", MainFreq);
                DataReady = 0;
            }
        }
    }
}

/*** (C) COPYRIGHT 2019 Nuvoton Technology Corp. ***/
