/******************************************************************************
 * @file     nuc505_isr.c
 * @brief    NUC505 series ISR source file
 *
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "NUC505Series.h"
#include "arm_math.h"
#include "config.h"

/*---------------------------------------------------------------------------*/
/* Global variables                                                          */
/*---------------------------------------------------------------------------*/
extern int16_t PcmRxBuff[2][BUFF_LEN * 2];
extern int16_t PcmTxBuff[2][BUFF_LEN * 2];
extern float32_t RawData[BUFF_LEN * 2];
extern uint8_t DataReady;
volatile static uint32_t s_flag1;
/*---------------------------------------------------------------------------*/
/* Functions                                                                 */
/*---------------------------------------------------------------------------*/
void I2S_IRQHandler(void)
{
    uint32_t u32I2SIntFlag;
    uint16_t i;

    /* Check interrupt flag */
    u32I2SIntFlag = I2S_GET_INT_FLAG(I2S, (I2S_STATUS_RDMATIF_Msk | I2S_STATUS_RDMAEIF_Msk));

    /* Copy RX data to TX buffer */
    if (u32I2SIntFlag & I2S_STATUS_RDMATIF_Msk) {

        I2S_CLR_INT_FLAG(I2S, I2S_STATUS_RDMATIF_Msk);

        /* Copy RX data to TX buffer */
        for (i = 0; i < BUFF_LEN; i++) {
            PcmTxBuff[0][i * 2] = PcmRxBuff[0][i * 2];
            PcmTxBuff[0][i * 2 + 1] = PcmRxBuff[0][i * 2 + 1];
            /* Copy data to working buffer */
            RawData[i * 2] = PcmRxBuff[0][i * 2]; //real part
            RawData[i * 2 + 1] = 0; //imaginary part
        }

        /* Working buffer data is ready */
        DataReady = 1;

    } else if (u32I2SIntFlag & I2S_STATUS_RDMAEIF_Msk) {

        I2S_CLR_INT_FLAG(I2S, I2S_STATUS_RDMAEIF_Msk);

        if (s_flag1 == 0) {
            s_flag1 = 1;
            /* Eable TX -> start to play */
            I2S_ENABLE_TXDMA(I2S);
            I2S_ENABLE_TX(I2S);
        }

        /* Copy RX data to TX buffer */
        for (i = 0; i < BUFF_LEN; i++) {
            PcmTxBuff[1][i * 2] = PcmRxBuff[1][i * 2];
            PcmTxBuff[1][i * 2 + 1] = PcmRxBuff[1][i * 2 + 1];
        }
    }
}

/*** (C) COPYRIGHT 2019 Nuvoton Technology Corp. ***/
